
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "lcddriver.h"



#define _lcd_SetCGRAMAddress(byte) _lcd_WriteToDisplay(((byte)&0x3F)|0x40,COM)


#define IE	(1<<0)
#define IWR	(1<<1)
#define IRS	(1<<2)

#define COM  0
#define DAN  1


//   

const unsigned  char __attribute__ ((progmem)) intansi[0x10]={	
																'0','1','2','3','4','5','6','7',
																'8','9'};
		
unsigned char IntToStr(int value, char* buff, unsigned char lenbuff)
					//,   ,   
{
	unsigned int 	conv_value	= value & 0b0111111111111111;//      
	char*			conv_str	= buff+lenbuff-1;	//    
	unsigned char	len			= 0;

	if(value)
	{
		for(;(conv_value)&&lenbuff;len++,conv_str--,lenbuff--)
		{
			*conv_str	=  pgm_read_byte(&intansi[conv_value%10]);
			conv_value	= (int)(conv_value/10);
		};
		if(value<0)
		{
		*conv_str = '-';
		};
		return len;
	}
	else
	{
	*conv_str = '0';
		return 1;
	}
}


unsigned char _lcd_read4_byte(void);
void _lcd_write4_byte(unsigned char byte);


volatile void Sleep(long int var)
{
  while(var--);
}



void _lcd_Display_Init(void)
{	
	_delay_ms(15);

	PORTB	&= ~IWR;
	PORTB	&= ~IRS;
	DDRB 	=  0xFF;
	PORTB	&= 0x0F;
	_delay_ms(15);

	//   
	PORTB	|= 0x30;
	PORTB	|= IE;
	_delay_us(10);

	PORTB	&= ~IE;
	_delay_ms(5);


	PORTB	|= IE;
	_delay_us(10);

	PORTB	&= ~IE;
	_delay_us(200);

	PORTB	|= IE;
	_delay_us(10);

	PORTB	&= ~IE;
	_delay_us(200);


	PORTB	&= 0x0F;
	PORTB	|= 0x20;
	PORTB	|= IE;
	_delay_us(10);

	PORTB&=~IE;
	_delay_us(200);

	_lcd_WriteToDisplay(0x28,COM);			// 
	_lcd_WriteToDisplay(0x0C,COM);			//   
	_lcd_WriteToDisplay(0x06,COM);			

	_lcd_ClrScr();

	_delay_ms(100);
}



void _lcd_SetPos(unsigned char x, unsigned char y)
{
	switch (y){
		case 1:	y = 0x40;	break;
	}
	_lcd_WriteToDisplay((y+x)|0x80, COM);
}


/////////////////////////////////////////////////////////////////////////////////////
void _lcd_WriteToDisplay(unsigned char byte,unsigned char DatCom)
{
	unsigned char 		wait_count_p	= WCLCDWB_P;
	unsigned long int	wait_count;

	for(;wait_count_p;wait_count_p--)
	{	
		if(DatCom)
			PORTB	|= IRS;
		else 
			PORTB	&= ~IRS;
	
		_lcd_write4_byte(byte);	

		PORTB	&= ~IRS;

		for(wait_count=WCLCDWB;wait_count;wait_count--)
	   		if(_lcd_read4_byte()&0x80) 
				return;
	}
}
//////////////////////////////////////////////////////////////////////////////////////

void _lcd_Print(char * str)
{
	for(;*str;str++)
		_lcd_WriteDataByteToDisplayRAM(*str);
}
///////////////////////////////////////////////////////////////////////////////////////

void _lcd_Out(char * data, unsigned char len)
{
	for(;len>0;len--, data++)
		_lcd_WriteDataByteToDisplayRAM(*data);
}
///////////////////////////////////////////////////////////////////////////////////////

void _lcd_ClrScr(void)
{
	_lcd_WriteToDisplay( 0x01, COM );
  	_delay_us(3000);
}
///////////////////////////////////////////////////////////////////////////////////////

void _lcd_LoadGraphSymbols(char const * graphic, unsigned char pos)
{
	unsigned char k;

	_lcd_SetCGRAMAddress(pos*8);
	unsigned char i	= 0;
	for(;i<8;i++)
		{
		k = pgm_read_byte(&graphic[i]);
		_lcd_WriteToDisplay(k, DAN);
		}

}
///////////////////////////////////////////////////////////////////////////////////////

void _lcd_write4_byte(unsigned char byte)
{
	_delay_us(5);///////////////////////////////
	DDRB	|= 0xFF;
	PORTB	&= ~IWR;
	_delay_us(5);///////////////////////////////
	PORTB	&= 0x0F;
	_delay_us(5);///////////////////////////////
	PORTB	|= byte&0xF0;
	_delay_us(5);///////////////////////////////
	PORTB	|= IE;
	_delay_us(5);///////////////////////////////
	PORTB	&=~IE;
	_delay_us(5);///////////////////////////////
	PORTB	&= 0x0F;
	_delay_us(5);///////////////////////////////
	PORTB	|= (byte<<4)&0xF0;
	_delay_us(5);///////////////////////////////
	PORTB	|= IE;
	_delay_us(5);///////////////////////////////
	PORTB	&=~IE;
	_delay_us(5);///////////////////////////////
}
///////////////////////////////////////////////////////////////////////////////////////

unsigned char _lcd_read4_byte(void)
{
  unsigned char result = 0;
	_delay_us(5);///////////////////////////////
	DDRB	&= 0x0F;
	PORTB	|= 0xF0;
	_delay_us(5);///////////////////////////////
	PORTB	|= IWR;
	_delay_us(10);///////////////////////////////
	PORTB	|= IE;
	Sleep(10);
	result	|= PINB&0xF0;
	PORTB	&=~IE;
	_delay_us(5);///////////////////////////////
	PORTB	|=IE;
	Sleep(10);
	result	|= (PINB&0xF0)>>4;
	PORTB	&=~IE;
	_delay_us(5);///////////////////////////////
	return result;
}
